from ZODB.POSException import ConflictError
from five import grok
from OFS.interfaces import IItem
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from plone.indexer import indexer

from .dmsdocument import IDmsDocument


@indexer(IDmsDocument)
def document_dynamic_searchable_text_indexer(obj):
    indexed_elements = [obj.title]

    # if there is no path to text/plain, do nothing
    transforms = getToolByName(obj, 'portal_transforms')

    had_version = False
    for child in reversed(obj.values()):
        if child.portal_type in ('dmsmainfile', 'dmsappendixfile'):
            if not child.file or child.file.getSize() == 0:
                continue

            if not transforms._findPath(child.file.contentType, 'text/plain'):
                continue

            # only index the latest version
            if child.portal_type == 'dmsmainfile' and had_version:
                continue
            had_version = True

            # convert it to text/plain
            try:
                datastream = transforms.convertTo(
                    'text/plain', child.file.data, mimetype=child.file.contentType,
                    filename=child.file.filename)
                indexed_elements.append(unicode(datastream.getData(), 'utf-8'))
            except (ConflictError, KeyboardInterrupt):
                raise

    return u' '.join(indexed_elements)

grok.global_adapter(document_dynamic_searchable_text_indexer,
                    name='SearchableText')


@indexer(IItem)
def treating_groups_indexer(obj):
    # skip acquisition for contained elements. None is considered as ()
    if base_hasattr(obj, 'treating_groups'):
        return obj.treating_groups


@indexer(IItem)
def recipient_groups_indexer(obj):
    # skip acquisition for contained elements. None is considered as ()
    if base_hasattr(obj, 'recipient_groups'):
        return obj.recipient_groups



# -*- coding: utf8 -*-

import unittest

from ecreall.helpers.testing.base import BaseTest

from collective.dms.basecontent.testing import DMS_TESTS_PROFILE_FUNCTIONAL
from ..source import RecipientGroupsVocabulary, TreatingGroupsVocabulary


class TestDmsdocument(unittest.TestCase, BaseTest):
    """Tests adapters"""

    layer = DMS_TESTS_PROFILE_FUNCTIONAL

    def setUp(self):
        super(TestDmsdocument, self).setUp()
        self.portal = self.layer['portal']

    def test_RecipientGroupsVocabulary(self):
        voc_inst = RecipientGroupsVocabulary()
        voc_ids = [i.token for i in voc_inst(self.portal).__iter__()]
        self.assertEqual(set(voc_ids), set(['Administrators',
                                             'Reviewers', 'Site Administrators']))
        voc_ids = [i.token for i in voc_inst(self.portal).search('user')]
        self.assertEqual(set(voc_ids), set(['test_user_1_', 'AuthenticatedUsers']))

    def test_TreatingGroupsVocabulary(self):
        voc_inst = TreatingGroupsVocabulary()
        voc_ids = [i.token for i in voc_inst(self.portal).__iter__()]
        self.assertEqual(set(voc_ids), set(['Administrators',
                                             'Reviewers', 'Site Administrators']))
        voc_ids = [i.token for i in voc_inst(self.portal).search('user')]
        self.assertEqual(set(voc_ids), set(['test_user_1_', 'AuthenticatedUsers']))



from five import grok

from plone.app.layout.viewlets.interfaces import IHtmlHeadLinks

from collective.dms.basecontent.dmsdocument import IDmsDocument
from collective.dms.basecontent import _
from collective.dms.basecontent.browser.listing import (VersionsTable,
                                                        DmsAppendixTable)
from collective.dms.basecontent.browser.table import TableViewlet

from zope.interface import Interface



grok.templatedir('templates')
grok.context(IDmsDocument)


class DmsAboveContentViewletManager(grok.ViewletManager):
    grok.name('dms.abovecontent')


class DmsBelowContentViewletManager(grok.ViewletManager):
    grok.name('dms.belowcontent')


class BaseViewlet(TableViewlet):
    grok.baseclass()
    grok.viewletmanager(DmsBelowContentViewletManager)
    __table__ = VersionsTable

    def contentFilter(self):
        return {'portal_type': self.portal_type,
                }


class VersionsViewlet(BaseViewlet):
    grok.name('dms.files')
    grok.template('versionsviewlet')
    grok.viewletmanager(DmsAboveContentViewletManager)
    grok.order(10)
    portal_type = 'dmsmainfile'
    label = _(u"Versions")
    noresult_message = _(u"There is no version note for this document.")

    def contentFilter(self):
        return {'portal_type': self.portal_type,
                'sort_on': 'getObjPositionInParent',
                'sort_order': 'descending'}


class AppendixViewlet(BaseViewlet):
    grok.name('dms.appendix')
    grok.order(20)
    portal_type = 'dmsappendixfile'
    label = _(u"Appendix")
    noresult_message = _(u"There is no appendix for this document.")
    __table__ = DmsAppendixTable

    def contentFilter(self):
        return {'portal_type': self.portal_type,
                'sort_on': 'getObjPositionInParent',
                'sort_order': 'ascending'}


class ChangeTitleViewlet(grok.Viewlet):
    grok.name('dms.changetitle')
    grok.context(Interface)
    grok.viewletmanager(IHtmlHeadLinks)
    grok.require('zope2.View')

    def render(self):
        return u"""
<script type="text/javascript">
    (function ($) {
        $(document).ready(function () {
            $("body.template-dmsincomingmail label[for='form-widgets-IDublinCore-title']").text('Objet');
            $("body.template-dmsoutgoingmail label[for='form-widgets-IDublinCore-title']").text('Objet');
            $("body.template-edit.portaltype-dmsincomingmail label[for='form-widgets-IDublinCore-title']").text('Objet');
            $("body.template-edit.portaltype-dmsoutgoingmail label[for='form-widgets-IDublinCore-title']").text('Objet');
        });
    })(jQuery);
</script>"""



# -*- coding: utf8 -*-

from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import FunctionalTesting

import collective.dms.basecontent

DMS = PloneWithPackageLayer(
    zcml_filename="testing.zcml",
    zcml_package=collective.dms.basecontent,
    gs_profile_id='collective.dms.basecontent:testing',
    name="DMS")

DMS_TESTS_PROFILE = PloneWithPackageLayer(
    bases=(DMS, ),
    zcml_filename="testing.zcml",
    zcml_package=collective.dms.basecontent,
    gs_profile_id='collective.dms.basecontent:testing',
    name="DMS_TESTS_PROFILE")

DMS_TESTS_PROFILE_FUNCTIONAL = FunctionalTesting(
    bases=(DMS_TESTS_PROFILE,), name="DMS_TESTS_PROFILE_FUNCTIONAL")




from five import grok
from z3c.table.interfaces import IBatchProvider
from zope.interface import Interface
import zope.publisher.interfaces.browser
import z3c.table.interfaces
from ZTUtils import make_query


class PloneBatchProvider(grok.MultiAdapter):
    grok.name('plonebatch')
    grok.implements(IBatchProvider)
    grok.adapts(
        Interface,
        zope.publisher.interfaces.browser.IBrowserRequest,
        z3c.table.interfaces.ITable)
    batchformkeys = None
    defaultBatchLinkCSS = u''
    batchLinkCSS = {'previous': u'previous',
                    'next': u'next',
                    'first': u'first',
                    'current': u'current',
                    'last': u'last'}
    batchSpacer = u'...'
    batchStartTag = u'<div class="listingBar">'
    batchEndTag = u'</div>'
    b_start_str = 'b_start'
    addNextAfterPrevious = True

    def __init__(self, context, request, table):
        self.__parent__ = context
        self.context = context
        self.request = request
        self.table = table
        self.batch = table.values

    def update(self):
        pass

    def render(self):
        self.update()
        batch = self.batch
#        if not batch.multiple_pages:
        if not (batch.sequence_length != batch.pagesize and \
                batch.sequence_length / batch.pagesize):
            return u""

        res = [self.batchStartTag]
        append = res.append
        if batch.has_previous:
            append(self.renderBatchLink(batch.previouspage,
                self.batchLinkCSS['previous'], text='&laquo;'))

        if self.addNextAfterPrevious and batch.has_next:
            append(self.renderBatchLink(batch.nextpage,
                self.batchLinkCSS['next'], text='&raquo;'))

        # Link to first
        if batch.show_link_to_first:
            append(self.renderBatchLink(1, self.batchLinkCSS['first']))
            if batch.second_page_not_in_navlist:
                append(self.batchSpacer)

        # Pagelist with links to previous pages for quick navigation
        for pagenumber in batch.previous_pages:
            append(self.renderBatchLink(pagenumber))

        # Current page
        append(self.renderCurrentBatchLink(batch.pagenumber,
            self.batchLinkCSS['current']))

        #  Pagelist with links to next pages for quick navigation
        for pagenumber in batch.next_pages:
            append(self.renderBatchLink(pagenumber))

        # Link to last
        if batch.show_link_to_last:
            if batch.before_last_page_not_in_navlist:
                append(self.batchSpacer)
            append(self.renderBatchLink(batch.lastpage,
                    self.batchLinkCSS['last']))

        if not self.addNextAfterPrevious and batch.has_next:
            append(self.renderBatchLink(batch.nextpage,
                self.batchLinkCSS['next'], text='&raquo;'))

        append(self.batchEndTag)
        return u'\n'.join(res)

    def renderCurrentBatchLink(self, pagenumber, cssClass=u''):
        return u'<span class="%s">%s</span>' % (cssClass, pagenumber)

    def renderBatchLink(self, pagenumber, cssClass=None, text=None):
        if text is None:
            text = pagenumber

        form = self.request.form
        start = max(pagenumber - 1, 0) * self.batch.pagesize
        cssClass = cssClass and \
                ' '.join([c for c in (self.defaultBatchLinkCSS, cssClass) if c]) or \
                self.defaultBatchLinkCSS
        if self.batchformkeys:
            batchlinkparams = dict([(key, form[key])
                                    for key in self.batchformkeys
                                    if key in form])
            query = make_query(batchlinkparams, {self.b_start_str: start})
            return u'<a data-query="%s"%s>%s</a>' % (
                query, cssClass and u' class="%s"' % cssClass or u'', text)
        else:
            batchlinkparams = form.copy()
            query = make_query(batchlinkparams, {self.b_start_str: start})
            url = '%s?%s' % (self.request.ACTUAL_URL, query)
            return u'<a href="%s"%s>%s</a>' % (
                    url, cssClass and u' class="%s"' % cssClass or u'', text)


class BootstrapPloneBatchProvider(PloneBatchProvider):
    grok.baseclass()
    batchLinkCSS = {'previous': u'previous',
                    'next': u'next',
                    'first': u'first',
                    'current': u'active',
                    'last': u'last'}
    batchSpacer = u'<li class="disabled"><a>&hellip;</a></li>'
    batchStartTag = u'<div class="pagination"><ul class="pager">'
    batchEndTag = u'</ul></div>'
    addNextAfterPrevious = False

    def renderBatchLink(self, pagenumber, cssClass=None, text=None):
        result = super(BootstrapPloneBatchProvider, self).renderBatchLink(
                pagenumber, u'', text)
        return u"""<li%s>%s</li>""" % (
                cssClass and u' class="%s"' % cssClass or u'', result)

    def renderCurrentBatchLink(self, pagenumber, cssClass=u''):
        return u'<li class="active"><a href="#">%s</a></li>' % pagenumber



#! -*- coding: utf8 -*-
from setuptools import setup, find_packages

version = '1.2'

long_description = (
    open('README.rst').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(name='collective.dms.basecontent',
      version=version,
      description="Base content types for document management system",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Ecreall, Entrouvert, IMIO',
      author_email='cedricmessiant@ecreall.com',
      url='https://github.com/collective/collective.dms.basecontent',
      license='gpl',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective', 'collective.dms'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'five.grok',
          'collective.documentviewer',
          'dexterity.localrolesfield',
          'plone.api',
          'plone.app.dexterity',
          'plone.directives.form',
          'plone.namedfile',
          'z3c.blobfile',
          'plone.app.contenttypes',
          'plone.app.relationfield',
          'plone.formwidget.contenttree',
          'plone.principalsource',
          'collective.z3cform.chosen',
          'z3c.table',
      ],
      extras_require={
          'test': ['plone.app.testing',
                   'ecreall.helpers.testing',
                   'plone.app.vocabularies'
                   ],
          },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import os.path
import Missing
from Acquisition import aq_base
from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName
from five import grok
from z3c.table import interfaces
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory
from zope.i18n import translate
import z3c.table.table
import z3c.table.column
from Products.CMFCore.WorkflowCore import WorkflowException
import plone.api

from collective.dms.basecontent import _

PMF = MessageFactory('plone')

grok.templatedir('templates')


class Column(z3c.table.column.Column, grok.MultiAdapter):
    grok.baseclass()
    grok.provides(interfaces.IColumn)


def get_value(item, attribute, default=None):
    try:
        value = getattr(aq_base(item), attribute)
        if value is Missing.Value:
            return default
    except AttributeError:
        obj = item.getObject()
        value = getattr(obj, attribute, default)

    if callable(value):
        value = value()

    return value


class DateColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        return self.table.format_date(value)


class DateTimeColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        return self.table.format_date(value, long_format=True)


class PrincipalColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute, default=())

        if not isinstance(value, (list, tuple)):
            value = (value,)

        gtool = getToolByName(plone.api.portal.get(), 'portal_groups')
        mtool = getToolByName(plone.api.portal.get(), 'portal_membership')
        principals = []
        for principal_id in value:
            user = mtool.getMemberById(principal_id)
            if user is not None:
                principals.append(user.getProperty('fullname', None) or user.getId())
            else:
                group = gtool.getGroupById(principal_id)
                if group is not None:
                    principals.append(group.getProperty('title', None) or group.getId())

        return ', '.join(principals).decode('utf-8')


class LinkColumn(z3c.table.column.LinkColumn, Column):
    grok.baseclass()

    def getLinkURL(self, item):
        """Setup link url."""
        if self.linkName is not None:
            return '%s/%s' % (item.getURL(), self.linkName)
        return item.getURL()


class TitleColumn(LinkColumn):
    grok.baseclass()
    header = PMF("Title")
    weight = 10

    def getLinkContent(self, item):
        title = get_value(item, 'Title')
        if isinstance(title, unicode):
            return title
        else:
            return unicode(title, 'utf-8', 'ignore')


class IconColumn(object):
    def getLinkContent(self, item):
        content = super(IconColumn, self).getLinkContent(item)
        return u"""<img title="%s" src="%s" />""" % (
                content,
                '%s/%s' % (self.table.portal_url, self.iconName))


class DeleteColumn(IconColumn, LinkColumn):
    grok.baseclass()
    header = u""
    weight = 9
    linkName = "delete_confirmation"
    linkContent = PMF('Delete')
    linkCSS = 'edm-delete-popup'
    iconName = "delete_icon.png"
    linkContent = PMF(u"Delete")

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        return sm.checkPermission('Delete objects', obj)

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(DeleteColumn, self).renderCell(item)


class DownloadColumn(IconColumn, LinkColumn):
    grok.baseclass()
    header = u""
    weight = 1
    linkName = "@@download"
    iconName = "download_icon.png"
    linkContent = _(u"Download file")


class ExternalEditColumn(IconColumn, LinkColumn):
    grok.baseclass()
    header = u""
    weight = 3
    linkName = "@@external_edit"
    iconName = "extedit_icon.png"
    linkContent = PMF(u"Edit with external application")

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        if not sm.checkPermission('Modify portal content', obj):
            return False

        if obj.file is None:
            return False

        ext = os.path.splitext(obj.file.filename)[-1].lower()
        if ext in (u'.pdf', u'.jpg', '.jpeg'):
            return False

        view = getMultiAdapter((obj, self.request), name='externalEditorEnabled')
        if not view.available():
            return False

        return True

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(ExternalEditColumn, self).renderCell(item)


class EditColumn(IconColumn, LinkColumn):
    grok.baseclass()
    header = u""
    weight = 2
    linkName = "edit"
    iconName = "edit.png"
    linkContent = PMF(u"Edit")
    linkCSS = 'overlay-form-reload'

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        return sm.checkPermission('Modify portal content', obj)

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(EditColumn, self).renderCell(item)


class StateColumn(Column):
    grok.baseclass()
    header = PMF(u"State")
    weight = 50

    def renderCell(self, item):
        try:
            wtool = self.table.wtool
            portal_type = get_value(item, 'portal_type')
            review_state = get_value(item, 'review_state')
            if not review_state:
                return u""
            state_title = wtool.getTitleForStateOnType(review_state,
                                                       portal_type)
            return translate(PMF(state_title), context=self.request)
        except WorkflowException:
            return u""


class LabelColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        if value is None:
            value = ''
        return value
